@@ -1,18 +1,20 @@
 """Calculation of the least upper bound types (joins)."""
 
 from collections import OrderedDict
-from typing import cast, List, Optional
+from typing import List, Optional
 
 from mypy.types import (
-    Type, AnyType, NoneTyp, TypeVisitor, Instance, UnboundType,
-    TypeVarType, CallableType, TupleType, TypedDictType, ErasedType, TypeList,
-    UnionType, FunctionLike, Overloaded, PartialType, DeletedType,
-    UninhabitedType, TypeType, true_or_false
+    Type, AnyType, NoneTyp, TypeVisitor, Instance, UnboundType, TypeVarType, CallableType,
+    TupleType, TypedDictType, ErasedType, UnionType, FunctionLike, Overloaded, LiteralType,
+    PartialType, DeletedType, UninhabitedType, TypeType, true_or_false, TypeOfAny,
 )
 from mypy.maptype import map_instance_to_supertype
-from mypy.subtypes import is_subtype, is_equivalent, is_subtype_ignoring_tvars, is_proper_subtype
-
-from mypy import experiments
+from mypy.subtypes import (
+    is_subtype, is_equivalent, is_subtype_ignoring_tvars, is_proper_subtype,
+    is_protocol_implementation, find_member
+)
+from mypy.nodes import ARG_NAMED, ARG_NAMED_OPT
+from mypy import state
 
 
 def join_simple(declaration: Optional[Type], s: Type, t: Type) -> Type:
@@ -99,7 +101,7 @@ class TypeJoinVisitor(TypeVisitor[Type]):
         self.s = s
 
     def visit_unbound_type(self, t: UnboundType) -> Type:
-        return AnyType()
+        return AnyType(TypeOfAny.special_form)
 
     def visit_union_type(self, t: UnionType) -> Type:
         if is_subtype(self.s, t):
@@ -111,11 +113,11 @@ class TypeJoinVisitor(TypeVisitor[Type]):
         return t
 
     def visit_none_type(self, t: NoneTyp) -> Type:
-        if experiments.STRICT_OPTIONAL:
+        if state.strict_optional:
             if isinstance(self.s, (NoneTyp, UninhabitedType)):
                 return t
             elif isinstance(self.s, UnboundType):
-                return AnyType()
+                return AnyType(TypeOfAny.special_form)
             else:
                 return UnionType.make_simplified_union([self.s, t])
         else:
@@ -138,13 +140,30 @@ class TypeJoinVisitor(TypeVisitor[Type]):
 
     def visit_instance(self, t: Instance) -> Type:
         if isinstance(self.s, Instance):
-            return join_instances(t, self.s)
+            nominal = join_instances(t, self.s)
+            structural = None  # type: Optional[Instance]
+            if t.type.is_protocol and is_protocol_implementation(self.s, t):
+                structural = t
+            elif self.s.type.is_protocol and is_protocol_implementation(t, self.s):
+                structural = self.s
+            # Structural join is preferred in the case where we have found both
+            # structural and nominal and they have same MRO length (see two comments
+            # in join_instances_via_supertype). Otherwise, just return the nominal join.
+            if not structural or is_better(nominal, structural):
+                return nominal
+            return structural
         elif isinstance(self.s, FunctionLike):
+            if t.type.is_protocol:
+                call = unpack_callback_protocol(t)
+                if call:
+                    return join_types(call, self.s)
             return join_types(t, self.s.fallback)
         elif isinstance(self.s, TypeType):
             return join_types(t, self.s)
         elif isinstance(self.s, TypedDictType):
             return join_types(t, self.s)
+        elif isinstance(self.s, LiteralType):
+            return join_types(t, self.s)
         else:
             return self.default(self.s)
 
@@ -160,8 +179,11 @@ class TypeJoinVisitor(TypeVisitor[Type]):
         elif isinstance(self.s, Overloaded):
             # Switch the order of arguments to that we'll get to visit_overloaded.
             return join_types(t, self.s)
-        else:
-            return join_types(t.fallback, self.s)
+        elif isinstance(self.s, Instance) and self.s.type.is_protocol:
+            call = unpack_callback_protocol(self.s)
+            if call:
+                return join_types(t, call)
+        return join_types(t.fallback, self.s)
 
     def visit_overloaded(self, t: Overloaded) -> Type:
         # This is more complex than most other cases. Here are some
@@ -210,6 +232,10 @@ class TypeJoinVisitor(TypeVisitor[Type]):
                 else:
                     return Overloaded(result)
             return join_types(t.fallback, s.fallback)
+        elif isinstance(s, Instance) and s.type.is_protocol:
+            call = unpack_callback_protocol(s)
+            if call:
+                return join_types(t, call)
         return join_types(t.fallback, s)
 
     def visit_tuple_type(self, t: TupleType) -> Type:
@@ -238,10 +264,19 @@ class TypeJoinVisitor(TypeVisitor[Type]):
             required_keys = set(items.keys()) & t.required_keys & self.s.required_keys
             return TypedDictType(items, required_keys, fallback)
         elif isinstance(self.s, Instance):
-            return join_instances(self.s, t.fallback)
+            return join_types(self.s, t.fallback)
         else:
             return self.default(self.s)
 
+    def visit_literal_type(self, t: LiteralType) -> Type:
+        if isinstance(self.s, LiteralType):
+            if t == self.s:
+                return t
+            else:
+                return join_types(self.s.fallback, t.fallback)
+        else:
+            return join_types(self.s, t.fallback)
+
     def visit_partial_type(self, t: PartialType) -> Type:
         # We only have partial information so we can't decide the join result. We should
         # never get here.
@@ -262,7 +297,7 @@ class TypeJoinVisitor(TypeVisitor[Type]):
         if isinstance(typ, Instance):
             return object_from_instance(typ)
         elif isinstance(typ, UnboundType):
-            return AnyType()
+            return AnyType(TypeOfAny.special_form)
         elif isinstance(typ, TupleType):
             return self.default(typ.fallback)
         elif isinstance(typ, TypedDictType):
@@ -272,7 +307,7 @@ class TypeJoinVisitor(TypeVisitor[Type]):
         elif isinstance(typ, TypeVarType):
             return self.default(typ.upper_bound)
         else:
-            return AnyType()
+            return AnyType(TypeOfAny.special_form)
 
 
 def join_instances(t: Instance, s: Instance) -> Type:
@@ -335,7 +370,6 @@ def is_similar_callables(t: CallableType, s: CallableType) -> bool:
     """Return True if t and s have identical numbers of
     arguments, default arguments and varargs.
     """
-
     return (len(t.arg_types) == len(s.arg_types) and t.min_args == s.min_args and
             t.is_var_arg == s.is_var_arg)
 
@@ -353,6 +387,7 @@ def join_similar_callables(t: CallableType, s: CallableType) -> CallableType:
     else:
         fallback = s.fallback
     return t.copy_modified(arg_types=arg_types,
+                           arg_names=combine_arg_names(t, s),
                            ret_type=join_types(t.ret_type, s.ret_type),
                            fallback=fallback,
                            name=None)
@@ -370,11 +405,42 @@ def combine_similar_callables(t: CallableType, s: CallableType) -> CallableType:
     else:
         fallback = s.fallback
     return t.copy_modified(arg_types=arg_types,
+                           arg_names=combine_arg_names(t, s),
                            ret_type=join_types(t.ret_type, s.ret_type),
                            fallback=fallback,
                            name=None)
 
 
+def combine_arg_names(t: CallableType, s: CallableType) -> List[Optional[str]]:
+    """Produces a list of argument names compatible with both callables.
+
+    For example, suppose 't' and 's' have the following signatures:
+
+    - t: (a: int, b: str, X: str) -> None
+    - s: (a: int, b: str, Y: str) -> None
+
+    This function would return ["a", "b", None]. This information
+    is then used above to compute the join of t and s, which results
+    in a signature of (a: int, b: str, str) -> None.
+
+    Note that the third argument's name is omitted and 't' and 's'
+    are both valid subtypes of this inferred signature.
+
+    Precondition: is_similar_types(t, s) is true.
+    """
+    num_args = len(t.arg_types)
+    new_names = []
+    named = (ARG_NAMED, ARG_NAMED_OPT)
+    for i in range(num_args):
+        t_name = t.arg_names[i]
+        s_name = s.arg_names[i]
+        if t_name == s_name or t.arg_kinds[i] in named or s.arg_kinds[i] in named:
+            new_names.append(t_name)
+        else:
+            new_names.append(None)
+    return new_names
+
+
 def object_from_instance(instance: Instance) -> Instance:
     """Construct the type 'builtins.object' from an instance type."""
     # Use the fact that 'object' is always the last class in the mro.
@@ -391,3 +457,10 @@ def join_type_list(types: List[Type]) -> Type:
     for t in types[1:]:
         joined = join_types(joined, t)
     return joined
+
+
+def unpack_callback_protocol(t: Instance) -> Optional[Type]:
+    assert t.type.is_protocol
+    if t.type.protocol_members == ['__call__']:
+        return find_member('__call__', t, t)
+    return None
