@@ -1,26 +1,25 @@
 """Test cases for mypy types and type operations."""
 
-from typing import List, Tuple
+from typing import List, Tuple, cast
 
-from mypy.myunit import (
-    Suite, assert_equal, assert_true, assert_false, assert_type
-)
+from mypy.test.helpers import Suite, assert_equal, assert_true, assert_false, assert_type, skip
 from mypy.erasetype import erase_type
 from mypy.expandtype import expand_type
 from mypy.join import join_types, join_simple
 from mypy.meet import meet_types
+from mypy.sametypes import is_same_type
 from mypy.types import (
     UnboundType, AnyType, CallableType, TupleType, TypeVarDef, Type, Instance, NoneTyp, Overloaded,
-    TypeType, UnionType, UninhabitedType, true_only, false_only, TypeVarId, TypeOfAny
+    TypeType, UnionType, UninhabitedType, true_only, false_only, TypeVarId, TypeOfAny, LiteralType
 )
-from mypy.nodes import ARG_POS, ARG_OPT, ARG_STAR, CONTRAVARIANT, INVARIANT, COVARIANT
+from mypy.nodes import ARG_POS, ARG_OPT, ARG_STAR, ARG_STAR2, CONTRAVARIANT, INVARIANT, COVARIANT
 from mypy.subtypes import is_subtype, is_more_precise, is_proper_subtype
 from mypy.test.typefixture import TypeFixture, InterfaceTypeFixture
+from mypy.state import strict_optional_set
 
 
 class TypesSuite(Suite):
-    def __init__(self) -> None:
-        super().__init__()
+    def setUp(self) -> None:
         self.x = UnboundType('X')  # Helpers
         self.y = UnboundType('Y')
         self.fx = TypeFixture()
@@ -93,7 +92,7 @@ class TypesSuite(Suite):
 
 
 class TypeOpsSuite(Suite):
-    def set_up(self) -> None:
+    def setUp(self) -> None:
         self.fx = TypeFixture(INVARIANT)
         self.fx_co = TypeFixture(COVARIANT)
         self.fx_contra = TypeFixture(CONTRAVARIANT)
@@ -153,11 +152,19 @@ class TypeOpsSuite(Suite):
 
     def test_erase_with_function_type(self) -> None:
         self.assert_erase(self.fx.callable(self.fx.a, self.fx.b),
-                          self.fx.callable_type(self.fx.nonet))
+                          CallableType(arg_types=[self.fx.anyt, self.fx.anyt],
+                                       arg_kinds=[ARG_STAR, ARG_STAR2],
+                                       arg_names=[None, None],
+                                       ret_type=self.fx.anyt,
+                                       fallback=self.fx.function))
 
     def test_erase_with_type_object(self) -> None:
         self.assert_erase(self.fx.callable_type(self.fx.a, self.fx.b),
-                          self.fx.callable_type(self.fx.nonet))
+                          CallableType(arg_types=[self.fx.anyt, self.fx.anyt],
+                                       arg_kinds=[ARG_STAR, ARG_STAR2],
+                                       arg_names=[None, None],
+                                       ret_type=self.fx.anyt,
+                                       fallback=self.fx.type_type))
 
     def test_erase_with_type_type(self) -> None:
         self.assert_erase(self.fx.type_a, self.fx.type_a)
@@ -240,6 +247,37 @@ class TypeOpsSuite(Suite):
         assert_false(is_proper_subtype(fx.gb, fx.ga))
         assert_false(is_proper_subtype(fx.ga, fx.gb))
 
+    def test_is_proper_subtype_and_subtype_literal_types(self) -> None:
+        fx = self.fx
+
+        lit1 = LiteralType(1, fx.a)
+        lit2 = LiteralType("foo", fx.d)
+        lit3 = LiteralType("bar", fx.d)
+
+        assert_true(is_proper_subtype(lit1, fx.a))
+        assert_false(is_proper_subtype(lit1, fx.d))
+        assert_false(is_proper_subtype(fx.a, lit1))
+        assert_true(is_proper_subtype(fx.uninhabited, lit1))
+        assert_false(is_proper_subtype(lit1, fx.uninhabited))
+        assert_true(is_proper_subtype(lit1, lit1))
+        assert_false(is_proper_subtype(lit1, lit2))
+        assert_false(is_proper_subtype(lit2, lit3))
+
+        assert_true(is_subtype(lit1, fx.a))
+        assert_false(is_subtype(lit1, fx.d))
+        assert_false(is_subtype(fx.a, lit1))
+        assert_true(is_subtype(fx.uninhabited, lit1))
+        assert_false(is_subtype(lit1, fx.uninhabited))
+        assert_true(is_subtype(lit1, lit1))
+        assert_false(is_subtype(lit1, lit2))
+        assert_false(is_subtype(lit2, lit3))
+
+        assert_false(is_proper_subtype(lit1, fx.anyt))
+        assert_false(is_proper_subtype(fx.anyt, lit1))
+
+        assert_true(is_subtype(lit1, fx.anyt))
+        assert_true(is_subtype(fx.anyt, lit1))
+
     # can_be_true / can_be_false
 
     def test_empty_tuple_always_false(self) -> None:
@@ -358,7 +396,7 @@ class TypeOpsSuite(Suite):
 
 
 class JoinSuite(Suite):
-    def set_up(self) -> None:
+    def setUp(self) -> None:
         self.fx = TypeFixture()
 
     def test_trivial_cases(self) -> None:
@@ -529,31 +567,29 @@ class JoinSuite(Suite):
         self.assert_join(ov(c(fx.a, fx.a), c(fx.b, fx.b)), c(any, fx.b), c(any, fx.b))
         self.assert_join(ov(c(fx.a, fx.a), c(any, fx.b)), c(fx.b, fx.b), c(any, fx.b))
 
+    @skip
     def test_join_interface_types(self) -> None:
-        self.skip()  # FIX
         self.assert_join(self.fx.f, self.fx.f, self.fx.f)
         self.assert_join(self.fx.f, self.fx.f2, self.fx.o)
         self.assert_join(self.fx.f, self.fx.f3, self.fx.f)
 
+    @skip
     def test_join_interface_and_class_types(self) -> None:
-        self.skip()  # FIX
-
         self.assert_join(self.fx.o, self.fx.f, self.fx.o)
         self.assert_join(self.fx.a, self.fx.f, self.fx.o)
 
         self.assert_join(self.fx.e, self.fx.f, self.fx.f)
 
+    @skip
     def test_join_class_types_with_interface_result(self) -> None:
-        self.skip()  # FIX
         # Unique result
         self.assert_join(self.fx.e, self.fx.e2, self.fx.f)
 
         # Ambiguous result
         self.assert_join(self.fx.e2, self.fx.e3, self.fx.anyt)
 
+    @skip
     def test_generic_interfaces(self) -> None:
-        self.skip()  # FIX
-
         fx = InterfaceTypeFixture()
 
         self.assert_join(fx.gfa, fx.gfa, fx.gfa)
@@ -587,6 +623,41 @@ class JoinSuite(Suite):
         self.assert_join(self.fx.type_type, self.fx.type_any, self.fx.type_type)
         self.assert_join(self.fx.type_b, self.fx.anyt, self.fx.anyt)
 
+    def test_literal_type(self) -> None:
+        a = self.fx.a
+        d = self.fx.d
+        lit1 = LiteralType(1, a)
+        lit2 = LiteralType(2, a)
+        lit3 = LiteralType("foo", d)
+
+        self.assert_join(lit1, lit1, lit1)
+        self.assert_join(lit1, a, a)
+        self.assert_join(lit1, d, self.fx.o)
+        self.assert_join(lit1, lit2, a)
+        self.assert_join(lit1, lit3, self.fx.o)
+        self.assert_join(lit1, self.fx.anyt, self.fx.anyt)
+        self.assert_join(UnionType([lit1, lit2]), lit2, UnionType([lit1, lit2]))
+        self.assert_join(UnionType([lit1, lit2]), a, a)
+        self.assert_join(UnionType([lit1, lit3]), a, UnionType([a, lit3]))
+        self.assert_join(UnionType([d, lit3]), lit3, UnionType([d, lit3]))
+        self.assert_join(UnionType([d, lit3]), d, UnionType([d, lit3]))
+        self.assert_join(UnionType([a, lit1]), lit1, UnionType([a, lit1]))
+        self.assert_join(UnionType([a, lit1]), lit2, UnionType([a, lit1]))
+        self.assert_join(UnionType([lit1, lit2]),
+                         UnionType([lit1, lit2]),
+                         UnionType([lit1, lit2]))
+
+        # The order in which we try joining two unions influences the
+        # ordering of the items in the final produced unions. So, we
+        # manually call 'assert_simple_join' and tune the output
+        # after swapping the arguments here.
+        self.assert_simple_join(UnionType([lit1, lit2]),
+                                UnionType([lit2, lit3]),
+                                UnionType([lit1, lit2, lit3]))
+        self.assert_simple_join(UnionType([lit2, lit3]),
+                                UnionType([lit1, lit2]),
+                                UnionType([lit2, lit3, lit1]))
+
     # There are additional test cases in check-inference.test.
 
     # TODO: Function types + varargs and default args.
@@ -628,7 +699,7 @@ class JoinSuite(Suite):
 
 
 class MeetSuite(Suite):
-    def set_up(self) -> None:
+    def setUp(self) -> None:
         self.fx = TypeFixture()
 
     def test_trivial_cases(self) -> None:
@@ -761,10 +832,8 @@ class MeetSuite(Suite):
         self.assert_meet(self.fx.e, self.fx.e2, self.fx.nonet)
         self.assert_meet(self.fx.e2, self.fx.e3, self.fx.nonet)
 
+    @skip
     def test_meet_with_generic_interfaces(self) -> None:
-        # TODO fix
-        self.skip()
-
         fx = InterfaceTypeFixture()
         self.assert_meet(fx.gfa, fx.m1, fx.m1)
         self.assert_meet(fx.gfa, fx.gfa, fx.gfa)
@@ -779,8 +848,31 @@ class MeetSuite(Suite):
         self.assert_meet(self.fx.type_type, self.fx.type_any, self.fx.type_any)
         self.assert_meet(self.fx.type_b, self.fx.anyt, self.fx.type_b)
 
+    def test_literal_type(self) -> None:
+        a = self.fx.a
+        d = self.fx.d
+        lit1 = LiteralType(1, a)
+        lit2 = LiteralType(2, a)
+        lit3 = LiteralType("foo", d)
+
+        self.assert_meet(lit1, lit1, lit1)
+        self.assert_meet(lit1, a, lit1)
+        self.assert_meet_uninhabited(lit1, lit3)
+        self.assert_meet_uninhabited(lit1, lit2)
+        self.assert_meet(UnionType([lit1, lit2]), lit1, lit1)
+        self.assert_meet(UnionType([lit1, lit2]), UnionType([lit2, lit3]), lit2)
+        self.assert_meet(UnionType([lit1, lit2]), UnionType([lit1, lit2]), UnionType([lit1, lit2]))
+        self.assert_meet(lit1, self.fx.anyt, lit1)
+        self.assert_meet(lit1, self.fx.o, lit1)
+
     # FIX generic interfaces + ranges
 
+    def assert_meet_uninhabited(self, s: Type, t: Type) -> None:
+        with strict_optional_set(False):
+            self.assert_meet(s, t, self.fx.nonet)
+        with strict_optional_set(True):
+            self.assert_meet(s, t, self.fx.uninhabited)
+
     def assert_meet(self, s: Type, t: Type, meet: Type) -> None:
         self.assert_simple_meet(s, t, meet)
         self.assert_simple_meet(t, s, meet)
@@ -807,3 +899,46 @@ class MeetSuite(Suite):
         return CallableType(list(a[:-1]),
                             [ARG_POS] * n, [None] * n,
                             a[-1], self.fx.function)
+
+
+class SameTypeSuite(Suite):
+    def setUp(self) -> None:
+        self.fx = TypeFixture()
+
+    def test_literal_type(self) -> None:
+        b = self.fx.b  # Reminder: b is a subclass of a
+        d = self.fx.d
+
+        lit1 = LiteralType(1, b)
+        lit2 = LiteralType(2, b)
+        lit3 = LiteralType("foo", d)
+
+        self.assert_same(lit1, lit1)
+        self.assert_same(UnionType([lit1, lit2]), UnionType([lit1, lit2]))
+        self.assert_same(UnionType([lit1, lit2]), UnionType([lit2, lit1]))
+        self.assert_not_same(lit1, b)
+        self.assert_not_same(lit1, lit2)
+        self.assert_not_same(lit1, lit3)
+
+        self.assert_not_same(lit1, self.fx.anyt)
+        self.assert_not_same(lit1, self.fx.nonet)
+
+    def assert_same(self, s: Type, t: Type, strict: bool = True) -> None:
+        self.assert_simple_is_same(s, t, expected=True, strict=strict)
+        self.assert_simple_is_same(t, s, expected=True, strict=strict)
+
+    def assert_not_same(self, s: Type, t: Type, strict: bool = True) -> None:
+        self.assert_simple_is_same(s, t, False, strict=strict)
+        self.assert_simple_is_same(t, s, False, strict=strict)
+
+    def assert_simple_is_same(self, s: Type, t: Type, expected: bool, strict: bool) -> None:
+        actual = is_same_type(s, t)
+        assert_equal(actual, expected,
+                     'is_same_type({}, {}) is {{}} ({{}} expected)'.format(s, t))
+
+        if strict:
+            actual2 = (s == t)
+            assert_equal(actual2, expected,
+                         '({} == {}) is {{}} ({{}} expected)'.format(s, t))
+            assert_equal(hash(s) == hash(t), expected,
+                         '(hash({}) == hash({}) is {{}} ({{}} expected)'.format(s, t))
